# Protocol Documentation
<a name="top"></a>

## Table of Contents

- [upstreamauthority.proto](#upstreamauthority.proto)
    - [MintX509CARequest](#spire.server.upstreamauthority.MintX509CARequest)
    - [MintX509CAResponse](#spire.server.upstreamauthority.MintX509CAResponse)
    - [PublishJWTKeyRequest](#spire.server.upstreamauthority.PublishJWTKeyRequest)
    - [PublishJWTKeyResponse](#spire.server.upstreamauthority.PublishJWTKeyResponse)
  
  
  
    - [UpstreamAuthority](#spire.server.upstreamauthority.UpstreamAuthority)
  

- [Scalar Value Types](#scalar-value-types)



<a name="upstreamauthority.proto"></a>
<p align="right"><a href="#top">Top</a></p>

## upstreamauthority.proto



<a name="spire.server.upstreamauthority.MintX509CARequest"></a>

### MintX509CARequest



| Field | Type | Label | Description |
| ----- | ---- | ----- | ----------- |
| csr | [bytes](#bytes) |  | Certificate signing request (PKCS#10) |
| preferred_ttl | [int32](#int32) |  | Preferred TTL is the TTL preferred by SPIRE server for signed CA. If zero, the plugin should determine its own TTL value. Plugins are free to ignore this and use their own policies around TTLs. |






<a name="spire.server.upstreamauthority.MintX509CAResponse"></a>

### MintX509CAResponse



| Field | Type | Label | Description |
| ----- | ---- | ----- | ----------- |
| x509_ca_chain | [bytes](#bytes) | repeated | Contains ASN.1 encoded certificates representing the X.509 CA along with any intermediates necessary to chain back to a certificate present in the upstream_x509_roots. |
| upstream_x509_roots | [bytes](#bytes) | repeated | The trusted X.509 root authorities for the upstream authority |






<a name="spire.server.upstreamauthority.PublishJWTKeyRequest"></a>

### PublishJWTKeyRequest



| Field | Type | Label | Description |
| ----- | ---- | ----- | ----------- |
| jwt_key | [spire.common.PublicKey](#spire.common.PublicKey) |  | The JWT signing key to publish upstream |






<a name="spire.server.upstreamauthority.PublishJWTKeyResponse"></a>

### PublishJWTKeyResponse



| Field | Type | Label | Description |
| ----- | ---- | ----- | ----------- |
| upstream_jwt_keys | [spire.common.PublicKey](#spire.common.PublicKey) | repeated | The upstream JWT signing keys |





 

 

 


<a name="spire.server.upstreamauthority.UpstreamAuthority"></a>

### UpstreamAuthority


| Method Name | Request Type | Response Type | Description |
| ----------- | ------------ | ------------- | ------------|
| MintX509CA | [MintX509CARequest](#spire.server.upstreamauthority.MintX509CARequest) | [MintX509CAResponse](#spire.server.upstreamauthority.MintX509CAResponse) stream | Mints an X.509 CA and responds with the signed X.509 CA certificate chain and upstream X.509 roots. If supported by the implementation, subsequent responses on the stream contain upstream X.509 root updates, otherwise the RPC is completed after sending the initial response.

Implementation note: The stream should be kept open open in the face of transient errors encountered while tracking changes to the upstream X.509 roots as SPIRE core will not reopen a closed stream until the next X.509 CA rotation. |
| PublishJWTKey | [PublishJWTKeyRequest](#spire.server.upstreamauthority.PublishJWTKeyRequest) | [PublishJWTKeyResponse](#spire.server.upstreamauthority.PublishJWTKeyResponse) stream | Publishes a JWT signing key upstream and responds with the upstream JWT keys. If supported by the implementation, subsequent responses on the stream contain upstream JWT key updates, otherwise the RPC is completed after sending the initial response.

This RPC is optional and will return NotImplemented if unsupported.

Implementation note: The stream should be kept open open in the face of transient errors encountered while tracking changes to the upstream JWT keys as SPIRE core will not reopen a closed stream until the next JWT key rotation. |
| Configure | [.spire.common.plugin.ConfigureRequest](#spire.common.plugin.ConfigureRequest) | [.spire.common.plugin.ConfigureResponse](#spire.common.plugin.ConfigureResponse) | Standard SPIRE plugin RPCs |
| GetPluginInfo | [.spire.common.plugin.GetPluginInfoRequest](#spire.common.plugin.GetPluginInfoRequest) | [.spire.common.plugin.GetPluginInfoResponse](#spire.common.plugin.GetPluginInfoResponse) |  |

 



## Scalar Value Types

| .proto Type | Notes | C++ Type | Java Type | Python Type |
| ----------- | ----- | -------- | --------- | ----------- |
| <a name="double" /> double |  | double | double | float |
| <a name="float" /> float |  | float | float | float |
| <a name="int32" /> int32 | Uses variable-length encoding. Inefficient for encoding negative numbers – if your field is likely to have negative values, use sint32 instead. | int32 | int | int |
| <a name="int64" /> int64 | Uses variable-length encoding. Inefficient for encoding negative numbers – if your field is likely to have negative values, use sint64 instead. | int64 | long | int/long |
| <a name="uint32" /> uint32 | Uses variable-length encoding. | uint32 | int | int/long |
| <a name="uint64" /> uint64 | Uses variable-length encoding. | uint64 | long | int/long |
| <a name="sint32" /> sint32 | Uses variable-length encoding. Signed int value. These more efficiently encode negative numbers than regular int32s. | int32 | int | int |
| <a name="sint64" /> sint64 | Uses variable-length encoding. Signed int value. These more efficiently encode negative numbers than regular int64s. | int64 | long | int/long |
| <a name="fixed32" /> fixed32 | Always four bytes. More efficient than uint32 if values are often greater than 2^28. | uint32 | int | int |
| <a name="fixed64" /> fixed64 | Always eight bytes. More efficient than uint64 if values are often greater than 2^56. | uint64 | long | int/long |
| <a name="sfixed32" /> sfixed32 | Always four bytes. | int32 | int | int |
| <a name="sfixed64" /> sfixed64 | Always eight bytes. | int64 | long | int/long |
| <a name="bool" /> bool |  | bool | boolean | boolean |
| <a name="string" /> string | A string must always contain UTF-8 encoded or 7-bit ASCII text. | string | String | str/unicode |
| <a name="bytes" /> bytes | May contain any arbitrary sequence of bytes. | string | ByteString | str |

